module IGraph.Internal.C2HS (

  -- * Conversion between C and Haskell types
  cIntConv, cDoubleConv, cToBool, cFromBool, cToEnum, cFromEnum,

  -- * Composite marshalling functions
  peekIntConv, peekFloatConv, peekBool

) where

-- system
import Control.Monad
import Foreign
import Foreign.C

-- Conversions -----------------------------------------------------------------
--

-- | Integral conversion
--
{-# INLINE cIntConv #-}
cIntConv :: (Integral a, Integral b) => a -> b
cIntConv  = fromIntegral

cDoubleConv :: CDouble -> Double
cDoubleConv  = realToFrac
{-# INLINE cDoubleConv #-}

-- | Obtain C value from Haskell 'Bool'.
--
{-# INLINE cFromBool #-}
cFromBool :: Num a => Bool -> a
cFromBool  = fromBool

-- | Obtain Haskell 'Bool' from C value.
--
{-# INLINE cToBool #-}
cToBool :: (Eq a, Num a) => a -> Bool
cToBool  = toBool

-- | Convert a C enumeration to Haskell.
--
{-# INLINE cToEnum #-}
cToEnum :: (Integral i, Enum e) => i -> e
cToEnum  = toEnum . cIntConv

-- | Convert a Haskell enumeration to C.
--
{-# INLINE cFromEnum #-}
cFromEnum :: (Enum e, Integral i) => e -> i
cFromEnum  = cIntConv . fromEnum

-- | Marshalling of numerals
--
peekIntConv :: (Storable a, Integral a, Integral b) => Ptr a -> IO b
peekIntConv = liftM cIntConv . peek
{-# INLINE peekIntConv #-}

peekBool :: (Storable a, Eq a, Num a)  => Ptr a -> IO Bool
peekBool = liftM cToBool . peek
{-# INLINE peekBool #-}

peekFloatConv :: Ptr CDouble -> IO Double
peekFloatConv = liftM cDoubleConv . peek
{-# INLINE peekFloatConv #-}
